==> ja-WiKicker-0.23.tbz: lib/perl5/site_perl/5.8.5/WiKicker/Log/SharedRecentLog.pm <==
#
# WiKicker
#
#  SharedRecentLog.pm
#
#
# Copyright (C) 2004 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: SharedRecentLog.pm,v 1.3 2004/04/02 12:18:29 naney Exp $
#
#

package WiKicker::Log::SharedRecentLog;

use 5.005;
use strict;
use Storable qw(freeze thaw);
use IPC::ShareLite;
use Carp;
use Fcntl ':flock';

use vars qw($MAGIC);

BEGIN {
  my @stat = stat(__FILE__);
  $MAGIC = $stat[9];
  croak "I can't get MAGIC" unless $MAGIC;
}

sub MAX_LOG {
  300;
}

sub new {
  my ($class, %options) = @_;

  unless ($options{ipc_key}) {
    $options{ipc_key} = undef;
  }
  my $self = bless { ipc_key => $options{ipc_key} }, $class;
}

sub get_log {
  my $self = shift;

  unless ($self->{ipc_key}) {
    return [];
  }

  my $share;
  eval {
    $share = $self->_get_share;
  };
  if ($@) {
    warn $@;
    return [];
  }

  my $data = $share->fetch;
  undef $share;

  my $value;
  if (defined $data && $data ne '') {
    $value = thaw($data);
  }
  else {
    warn "no log";
    return [];
  }
  unless (ref($value) eq "HASH" && $value->{magic} == $MAGIC) {
    warn "Unknown shared recent log data: " . ref($value) . ": " . $value->{magic};
    return [];
  }
  #-IFDEBUG: WiKicker::total_size("shared log", $value->{log});
  return $value->{log} || [];
}

sub add_log {
  my ($self, $user_id, $user_name, $request) = @_;

  unless ($self->{ipc_key}) {
    return;
  }

  my $share;
  eval {
    $share = $self->_get_share;
  };
  if ($@) {
    warn $@;
    return;
  }

  $share->lock(LOCK_EX);
  eval {
    my $data = $share->fetch;
    my $value;
    if (defined $data && $data ne '') {
      $value = thaw($data);
    }
    else {
      $value = { magic => $MAGIC, log => [] };
    }
    unless (ref($value) eq "HASH" && $value->{magic} == $MAGIC) {
      warn "Unknown shared recent log data: " . ref($value) . ": " . $value->{magic};
      $value = { magic => $MAGIC, log => [] };
    }
    my $log = $value->{log} || [];
    my $max_log = MAX_LOG;
    my $size = scalar(@$log);
    if ($size >= $max_log) {
      splice(@$log, 0, $size - $max_log + 1);
    }
    push(@$log, $self->_make_log_item($user_id, $user_name, $request));
    $value->{log} = $log;
    $share->store(freeze($value));
  };
  $share->unlock;
  undef $share;
}

sub _get_share {
  my $self = shift;

  return new IPC::ShareLite(-key => $self->{ipc_key},
                            -create => 'yes',
                            -destroy => 'no',
                            -size => 1024 * 512) || croak $!;
}

sub _delete_share {
  my $self = shift;
  my $share = new IPC::ShareLite(-key => $self->{ipc_key},
                                 -create => 'yes',
                                 -destroy => 'no',
                                 -size => 1024 * 512,
                                 -destroy => 'yes',
                                ) || croak $!;
  undef $share;
}

sub _make_log_item {
  my ($self, $user_id, $user_name, $request) = @_;

  my $now = time;
  return [ $now,
           $ENV{REMOTE_ADDR},
           $user_id,
           $user_name,
           $request,
           $ENV{REQUEST_URI},
           $ENV{HTTP_REFERER},
           $ENV{HTTP_USER_AGENT} ];
}

1;

==> ja-WiKicker-0.26.tbz: lib/perl5/site_perl/5.8.9/WiKicker/Log/SharedRecentLog.pm <==
#
# WiKicker
#
#  SharedRecentLog.pm
#
#
# Copyright (C) 2004 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: SharedRecentLog.pm,v 1.4 2004/08/03 12:55:52 naney Exp $
#
#

package WiKicker::Log::SharedRecentLog;

use 5.005;
use strict;
use Storable qw(freeze thaw);
use IPC::ShareLite;
use Carp;
use Fcntl ':flock';

use vars qw($MAGIC);

BEGIN {
  my @stat = stat(__FILE__);
  $MAGIC = $stat[9];
  croak q(I can't get MAGIC) unless $MAGIC;
}

sub MAX_LOG {
  300;
}

sub new {
  my ($class, %options) = @_;

  unless ($options{ipc_key}) {
    $options{ipc_key} = undef;
  }
  my $self = bless { ipc_key => $options{ipc_key} }, $class;
}

sub get_log {
  my $self = shift;

  unless ($self->{ipc_key}) {
    return [];
  }

  my $share;
  eval {
    $share = $self->_get_share;
  };
  if ($@) {
    warn $@;
    return [];
  }

  my $data = $share->fetch;
  undef $share;

  my $value;
  if (defined $data && $data ne '') {
    $value = thaw($data);
  }
  else {
    warn 'no log';
    return [];
  }
  unless (ref($value) eq 'HASH' && $value->{magic} == $MAGIC) {
    warn 'Unknown shared recent log data: ' . ref($value) . ': ' . $value->{magic};
    return [];
  }
  #-IFDEBUG: WiKicker::total_size("shared log", $value->{log});
  return $value->{log} || [];
}

sub add_log {
  my ($self, $user_id, $user_name, $request) = @_;

  unless ($self->{ipc_key}) {
    return;
  }

  my $share;
  eval {
    $share = $self->_get_share;
  };
  if ($@) {
    warn $@;
    return;
  }

  $share->lock(LOCK_EX);
  eval {
    my $data = $share->fetch;
    my $value;
    if (defined $data && $data ne '') {
      $value = thaw($data);
    }
    else {
      $value = { magic => $MAGIC, log => [] };
    }
    unless (ref($value) eq 'HASH' && $value->{magic} == $MAGIC) {
      warn 'Unknown shared recent log data: ' . ref($value) . ': ' . $value->{magic};
      $value = { magic => $MAGIC, log => [] };
    }
    my $log = $value->{log} || [];
    my $max_log = MAX_LOG;
    my $size = scalar(@$log);
    if ($size >= $max_log) {
      splice(@$log, 0, $size - $max_log + 1);
    }
    push(@$log, $self->_make_log_item($user_id, $user_name, $request));
    $value->{log} = $log;
    $share->store(freeze($value));
  };
  $share->unlock;
  undef $share;
}

sub _get_share {
  my $self = shift;

  return new IPC::ShareLite(-key => $self->{ipc_key},
                            -create => 'yes',
                            -destroy => 'no',
                            -size => 1024 * 512) || croak $!;
}

sub _delete_share {
  my $self = shift;
  my $share = new IPC::ShareLite(-key => $self->{ipc_key},
                                 -create => 'yes',
                                 -destroy => 'no',
                                 -size => 1024 * 512,
                                 -destroy => 'yes',
                                ) || croak $!;
  undef $share;
}

sub _make_log_item {
  my ($self, $user_id, $user_name, $request) = @_;

  my $now = time;
  return [ $now,
           $ENV{REMOTE_ADDR},
           $user_id,
           $user_name,
           $request,
           $ENV{REQUEST_URI},
           $ENV{HTTP_REFERER},
           $ENV{HTTP_USER_AGENT} ];
}

1;
